@@ -1,6 +1,6 @@
 /*
  *	aegis - project change supervisor
- *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1991-1999, 2001, 2002 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -32,7 +32,7 @@
 #include <ael/change/by_state.h>
 #include <arglex2.h>
 #include <change.h>
-#include <change_bran.h>
+#include <change/branch.h>
 #include <change/file.h>
 #include <col.h>
 #include <commit.h>
@@ -134,6 +134,7 @@ develop_begin_undo_main()
 	string_ty	*usr_name;
 
 	trace(("develop_begin_undo_main()\n{\n"/*}*/));
+	arglex();
 	project_name = 0;
 	change_number = 0;
 	usr_name = 0;
@@ -167,7 +168,7 @@ develop_begin_undo_main()
 				sub_context_ty	*scp;
 
 				scp = sub_context_new();
-				sub_var_set(scp, "Number", "%ld", change_number);
+				sub_var_set_long(scp, "Number", change_number);
 				fatal_intl(scp, i18n("change $number out of range"));
 				/* NOTREACHED */
 				sub_context_delete(scp);
@@ -351,6 +352,11 @@ develop_begin_undo_main()
 	commit();
 	lock_release();
 
+	/*
+	 * run the notification command
+	 */
+	change_run_develop_begin_undo_command(cp, up);
+
 	/*
 	 * verbose success message
 	 */
@@ -367,20 +373,13 @@ develop_begin_undo_main()
 void
 develop_begin_undo()
 {
-	trace(("develop_begin_undo()\n{\n"/*}*/));
-	switch (arglex())
+	static arglex_dispatch_ty dispatch[] =
 	{
-	default:
-		develop_begin_undo_main();
-		break;
+		{ arglex_token_help,		develop_begin_undo_help, },
+		{ arglex_token_list,		develop_begin_undo_list, },
+	};
 
-	case arglex_token_help:
-		develop_begin_undo_help();
-		break;
-
-	case arglex_token_list:
-		develop_begin_undo_list();
-		break;
-	}
-	trace((/*{*/"}\n"));
+	trace(("develop_begin_undo()\n{\n"));
+	arglex_dispatch(dispatch, SIZEOF(dispatch), develop_begin_undo_main);
+	trace(("}\n"));
 }
