@@ -1,6 +1,6 @@
 /*
  *	aegis - project change supervisor
- *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1991-1999, 2001, 2002 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@
 #include <ael/change/files.h>
 #include <aermu.h>
 #include <arglex2.h>
-#include <change_bran.h>
+#include <change/branch.h>
 #include <change/file.h>
 #include <commit.h>
 #include <error.h>
@@ -150,7 +150,7 @@ remove_file_undo_list()
 				sub_context_ty	*scp;
 
 				scp = sub_context_new();
-				sub_var_set(scp, "Number", "%ld", change_number);
+				sub_var_set_long(scp, "Number", change_number);
 				fatal_intl(scp, i18n("change $number out of range"));
 				/* NOTREACHED */
 				sub_context_delete(scp);
@@ -213,6 +213,7 @@ remove_file_undo_main()
 	string_ty	*base;
 
 	trace(("remove_file_undo_main()\n{\n"/*}*/));
+	arglex();
 	project_name = 0;
 	change_number = 0;
 	string_list_constructor(&wl);
@@ -253,7 +254,7 @@ remove_file_undo_main()
 				sub_context_ty	*scp;
 
 				scp = sub_context_new();
-				sub_var_set(scp, "Number", "%ld", change_number);
+				sub_var_set_long(scp, "Number", change_number);
 				fatal_intl(scp, i18n("change $number out of range"));
 				/* NOTREACHED */
 				sub_context_delete(scp);
@@ -400,7 +401,7 @@ remove_file_undo_main()
 			sub_context_ty	*scp;
 
 			scp = sub_context_new();
-			sub_var_set(scp, "File_Name", "%S", wl.string[j]);
+			sub_var_set_string(scp, "File_Name", wl.string[j]);
 			change_error(cp, scp, i18n("$filename unrelated"));
 			sub_context_delete(scp);
 			++number_of_errors;
@@ -432,7 +433,7 @@ remove_file_undo_main()
 						sub_context_ty	*scp;
 
 						scp = sub_context_new();
-						sub_var_set(scp, "File_Name", "%S", s3);
+						sub_var_set_string(scp, "File_Name", s3);
 						change_error(cp, scp, i18n("too many $filename"));
 						sub_context_delete(scp);
 						++number_of_errors;
@@ -448,10 +449,10 @@ remove_file_undo_main()
 
 				scp = sub_context_new();
 				if (s2->str_length)
-					sub_var_set(scp, "File_Name", "%S", s2);
+					sub_var_set_string(scp, "File_Name", s2);
 				else
-					sub_var_set(scp, "File_Name", ".");
-				sub_var_set(scp, "Number", "%ld", (long)wl_in.nstrings);
+					sub_var_set_charstar(scp, "File_Name", ".");
+				sub_var_set_long(scp, "Number", (long)wl_in.nstrings);
 				sub_var_optional(scp, "Number");
 				change_error
 				(
@@ -470,7 +471,7 @@ remove_file_undo_main()
 				sub_context_ty	*scp;
 
 				scp = sub_context_new();
-				sub_var_set(scp, "File_Name", "%S", s2);
+				sub_var_set_string(scp, "File_Name", s2);
 				change_error(cp, scp, i18n("too many $filename"));
 				sub_context_delete(scp);
 				++number_of_errors;
@@ -502,8 +503,8 @@ remove_file_undo_main()
 				sub_context_ty	*scp;
 
 				scp = sub_context_new();
-				sub_var_set(scp, "File_Name", "%S", s1);
-				sub_var_set(scp, "Guess", "%S", c_src_data->file_name);
+				sub_var_set_string(scp, "File_Name", s1);
+				sub_var_set_string(scp, "Guess", c_src_data->file_name);
 				change_error
 				(
 					cp,
@@ -517,7 +518,7 @@ remove_file_undo_main()
 				sub_context_ty	*scp;
 
 				scp = sub_context_new();
-				sub_var_set(scp, "File_Name", "%S", s1);
+				sub_var_set_string(scp, "File_Name", s1);
 				change_error(cp, scp, i18n("no $filename"));
 				sub_context_delete(scp);
 			}
@@ -529,7 +530,7 @@ remove_file_undo_main()
 			sub_context_ty	*scp;
 
 			scp = sub_context_new();
-			sub_var_set(scp, "File_Name", "%S", s1);
+			sub_var_set_string(scp, "File_Name", s1);
 			change_error(cp, scp, i18n("bad rm undo $filename"));
 			sub_context_delete(scp);
 			++number_of_errors;
@@ -542,7 +543,7 @@ remove_file_undo_main()
 		sub_context_ty	*scp;
 
 		scp = sub_context_new();
-		sub_var_set(scp, "Number", "%d", number_of_errors);
+		sub_var_set_long(scp, "Number", number_of_errors);
 		sub_var_optional(scp, "Number");
 		change_fatal(cp, scp, i18n("remove file undo fail"));
 		/* NOTREACHED */
@@ -574,7 +575,9 @@ remove_file_undo_main()
 		s1 = wl.string[j];
 		if (mend_symlinks)
 		{
+			user_become_undo();
 			psrc_data = project_file_find(pp, s1);
+			user_become(up);
 			if
 			(
 				psrc_data
@@ -641,7 +644,7 @@ remove_file_undo_main()
 	 * run the change file command
 	 * and the project file command if necessary
 	 */
-	change_run_change_file_command(cp, &wl, up);
+	change_run_remove_file_undo_command(cp, &wl, up);
 	change_run_project_file_command(cp, up);
 
 	/*
@@ -659,7 +662,7 @@ remove_file_undo_main()
 		sub_context_ty	*scp;
 
 		scp = sub_context_new();
-		sub_var_set(scp, "File_Name", "%S", wl.string[j]);
+		sub_var_set_string(scp, "File_Name", wl.string[j]);
 		change_verbose(cp, scp, i18n("remove file undo $filename complete"));
 		sub_context_delete(scp);
 	}
@@ -687,20 +690,13 @@ remove_file_undo_main()
 void
 remove_file_undo()
 {
-	trace(("remove_file_undo()\n{\n"/*}*/));
-	switch (arglex())
+	static arglex_dispatch_ty dispatch[] =
 	{
-	default:
-		remove_file_undo_main();
-		break;
+		{ arglex_token_help,		remove_file_undo_help,	},
+		{ arglex_token_list,		remove_file_undo_list,	},
+	};
 
-	case arglex_token_help:
-		remove_file_undo_help();
-		break;
-
-	case arglex_token_list:
-		remove_file_undo_list();
-		break;
-	}
-	trace((/*{*/"}\n"));
+	trace(("remove_file_undo()\n{\n"));
+	arglex_dispatch(dispatch, SIZEOF(dispatch), remove_file_undo_main);
+	trace(("}\n"));
 }
