@@ -82,6 +82,7 @@ mangle_file_names(plp, pp)
 	int		npaths;
 	size_t		idx;
 	char		*cp;
+	string_ty	*dev_null;
 
 	/*
 	 * Look for the name with the fewest removed leading path
@@ -89,6 +90,7 @@ mangle_file_names(plp, pp)
 	 * project.
 	 */
 	trace(("mangle_file_names()\n{\n"));
+	dev_null = str_from_c("/dev/null");
 	best.npaths = 32767;
 	best.idx = 32767;
 	for (j = 0; j < plp->length; ++j)
@@ -97,6 +99,8 @@ mangle_file_names(plp, pp)
 		for (idx = 0; idx < p->name.nstrings; ++idx)
 		{
 			npaths = 0;
+			if (str_equal(dev_null, p->name.string[idx]))
+				continue;
 			cp = p->name.string[idx]->str_text;
 			for (;;)
 			{
@@ -129,6 +133,7 @@ mangle_file_names(plp, pp)
 			}
 		}
 	}
+	str_free(dev_null);
 	if (best.npaths == 32767)
 	{
 		best.npaths = 0;
@@ -330,6 +335,22 @@ receive()
 				goto too_many_trojans;
 			trojan = 0;
 			break;
+
+		case arglex_token_delta:
+			if (delta)
+				duplicate_option(usage);
+			switch (arglex())
+			{
+			default:
+				option_needs_number(arglex_token_delta, usage);
+				/*NOTREACHED*/
+			
+			case arglex_token_number:
+			case arglex_token_string:
+				delta = arglex_value.alv_string;
+				break;
+			}
+			break;
 		}
 		arglex();
 	}
