@@ -1,6 +1,6 @@
 /*
  *	aegis - project change supervisor
- *	Copyright (C) 1999 Peter Miller;
+ *	Copyright (C) 1999, 2001 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -34,11 +34,12 @@ str_quote_shell(s)
 	const char	*cp;
 	int		needs_quoting;
 	static stracc_t	buffer;
+	int		mode;
 
 	/*
 	 * Convert the work list to a single string.
 	 */
-	trace(("str_quote()\n{\n"/*}*/));
+	trace(("str_quote()\n{\n"));
 
 	/*
 	 * Work out if the string needs quoting.
@@ -49,9 +50,10 @@ str_quote_shell(s)
 	 * and remain empty.
 	 */
 	needs_quoting = 0;
+	mode = 0;
 	for (cp = s->str_text; *cp; ++cp)
 	{
-		if (isspace(*cp))
+		if (isspace((unsigned char)*cp))
 		{
 			needs_quoting = 1;
 			break;
@@ -61,7 +63,20 @@ str_quote_shell(s)
 		default:
 			continue;
 
-		case '!': case '"': case '#': case '$': case '&': case '\'':
+		case '!':
+			/* special for bash and csh */
+			if (!mode)
+				mode = '\'';
+			needs_quoting = 1;
+			break;
+
+		case '\'':
+			if (!mode)
+				mode = '"';
+			needs_quoting = 1;
+			break;
+
+		case '"': case '#': case '$': case '&':
 		case '(': case ')': case '*': case ':': case ';': case '<':
 		case '=': case '>': case '?': case '[': case '\\': case ']':
 		case '^': case '`': case '{': case '|': case '}': case '~':
@@ -78,42 +93,86 @@ str_quote_shell(s)
 	{
 		s = str_copy(s);
 		trace(("return %8.8lX;\n", (long)s));
-		trace((/*{*/"}\n"));
+		trace(("}\n"));
 		return s;
 	}
 
+	/*
+	 * If we have a choice, use single quote mode,
+	 * it's shorter and easier to read.
+	 */
+	if (!mode)
+		mode ='\'';
+
 	/*
 	 * Form the quoted string, using the minimum number of escapes.
 	 *
 	 * The gotcha here is the backquote: the `blah` substitution is
-	 * still active within double quotes.
+	 * still active within double quotes.  And so are a few others. 
+	 *
+	 * Also, there are some difficulties: the single quote can't be
+	 * quoted within single quotes, and the exclamation mark can't
+	 * be quoted by anything *except* single quotes.  Sheesh.
+	 *
+	 * Also, the rules change depending on which style of quoting
+	 * is in force at the time.
 	 */
 	stracc_open(&buffer);
-	stracc_char(&buffer, '"');
+	stracc_char(&buffer, mode);
 	for (cp = s->str_text; *cp; ++cp)
 	{
-		switch (*cp)
+		if (mode == '\'')
 		{
-		case '\n':
-		case '!':
-		case '"':
-		case '\\':
-		case '`':
-			stracc_char(&buffer, '\\');
-			/* fall through... */
+			/* within single quotes */
+			if (*cp == '\'')
+			{
+				/*
+				 * You can't quote a single quote within
+				 * single quotes.  Need to change to
+				 * double quote mode.
+				 */
+				stracc_chars(&buffer, "'\"'", 3);
+				mode = '"';
+			}
+			else
+				stracc_char(&buffer, *cp);
+		}
+		else
+		{
+			/* within double quotes */
+			switch (*cp)
+			{
+			case '!':
+				/*
+				 * You can't quote an exclamation mark
+				 * within double quotes.  Need to change
+				 * to single quote mode.
+				 */
+				stracc_chars(&buffer, "\"'!", 3);
+				mode = '\'';
+				break;
 
-		default:
-			stracc_char(&buffer, *cp);
-			break;
+			case '\n':
+			case '"':
+			case '\\':
+			case '`': /* stop command substitutions */
+			case '$': /* stop variable substitutions */
+				stracc_char(&buffer, '\\');
+				/* fall through... */
+
+			default:
+				stracc_char(&buffer, *cp);
+				break;
+			}
 		}
 	}
-	stracc_char(&buffer, '"');
+	stracc_char(&buffer, mode);
 	s = stracc_close(&buffer);
 
 	/*
 	 * all done
 	 */
 	trace(("return %8.8lX;\n", (long)s));
-	trace((/*{*/"}\n"));
+	trace(("}\n"));
 	return s;
 }
