@@ -41,7 +41,7 @@ str_trim(s)
 	language_human();
 	for (cp = s->str_text; *cp; ++cp)
 	{
-		if (isspace(*cp))
+		if (isspace((unsigned char)*cp))
 			++whitespace;
 		else
 		{
