@@ -1,6 +1,6 @@
 /*
  *	aegis - project change supervisor
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
+ *	Copyright (C) 1991-1994, 1997-1999, 2001 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -202,16 +202,17 @@ gen_code(type)
 	indent_more();
 	indent_printf("return;\n");
 	indent_less();
-	indent_printf("output_fprintf(fp, \"%%s = \", name);\n");
+	indent_printf("output_fputs(fp, name);\n");
+	indent_printf("output_fputs(fp, \" = \");\n");
 	indent_printf(/*{*/"}\n");
 	indent_printf
 	(
-		"output_fprintf(fp, \"%%s\", %s_s[this]);\n",
+		"output_fputs(fp, %s_s[this]);\n",
 		this->name->str_text
 	);
 	indent_printf("if (name)\n");
 	indent_more();
-	indent_printf("output_fprintf(fp, \";\\n\");\n");
+	indent_printf("output_fputs(fp, \";\\n\");\n");
 	indent_less();
 	indent_printf(/*{*/"}\n");
 
@@ -300,6 +301,16 @@ gen_code(type)
 	indent_printf("%s\1*this;\n", "void");
 	indent_less();
 	indent_printf("{\n"/*}*/);
+	indent_printf("if (!%s_f[0])\n", this->name->str_text);
+	indent_more();
+	indent_printf
+	(
+		"slow_to_fast(%s_s, %s_f, SIZEOF(%s_s));\n",
+		this->name->str_text,
+		this->name->str_text,
+		this->name->str_text
+	);
+	indent_less();
 	indent_printf("return\n");
 	indent_more();
 	indent_printf("generic_enum_convert\n");
