@@ -1,6 +1,6 @@
 /*
  *	aegis - project change supervisor
- *	Copyright (C) 1999 Peter Miller;
+ *	Copyright (C) 1999, 2001 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -31,21 +31,23 @@
 #include <col.h>
 #include <error.h> /* for assert */
 #include <option.h>
+#include <output.h>
 #include <project.h>
 #include <project/file.h>
 #include <project_hist.h>
 #include <str.h>
+#include <str_list.h>
 #include <trace.h>
 #include <user.h>
 
 
-static void showtime _((int, time_t, int));
+static void showtime _((output_ty *, time_t, int));
 
 static void
-showtime(colnum, when, exempt)
-	int	colnum;
-	time_t	when;
-	int	exempt;
+showtime(fp, when, exempt)
+	output_ty	*fp;
+	time_t		when;
+	int		exempt;
 {
 	if (when)
 	{
@@ -53,13 +55,13 @@ showtime(colnum, when, exempt)
 		char		buffer[100];
 
 		tm = localtime(&when);
-		strftime(buffer, sizeof(buffer), "%H:%M:%S %d-%b-%y", tm);
-		col_puts(colnum, buffer);
+		strftime(buffer, sizeof(buffer), "%H:%M:%S %d-%b-%Y", tm);
+		output_fputs(fp, buffer);
 	}
 	else if (exempt)
-		col_puts(colnum, "exempt");
+		output_fputs(fp, "exempt");
 	else
-		col_puts(colnum, "required");
+		output_fputs(fp, "required");
 }
 
 
@@ -69,14 +71,15 @@ list_change_details(project_name, change_number)
 	long		change_number;
 {
 	int		j;
-	int		head_col;
-	int		body_col;
+	output_ty	*head_col;
+	output_ty	*body_col;
 	project_ty	*pp;
 	change_ty	*cp;
 	cstate		cstate_data;
 	user_ty		*up;
 	string_ty	*line1;
 	int		left;
+	col_ty		*colp;
 
 	/*
 	 * locate project data
@@ -107,7 +110,7 @@ list_change_details(project_name, change_number)
 	/*
 	 * identification
 	 */
-	col_open((char *)0);
+	colp = col_open((string_ty *)0);
 	line1 =
 		str_format
 		(
@@ -115,54 +118,54 @@ list_change_details(project_name, change_number)
 			project_name_get(pp),
 			magic_zero_decode(change_number)
 		);
-	col_title(line1->str_text, "Change Details");
+	col_title(colp, line1->str_text, "Change Details");
 	str_free(line1);
 
 	/* the heading columns is the whole page wide */
-	head_col = col_create(0, 0);
+	head_col = col_create(colp, 0, 0, (const char *)0);
 
 	/* the body columns is indented */
-	body_col = col_create(INDENT_WIDTH, 0);
-	col_puts(head_col, "NAME");
-	col_eoln();
-	col_printf(body_col, "Project \"%s\"", project_name_get(pp)->str_text);
+	body_col = col_create(colp, INDENT_WIDTH, 0, (const char *)0);
+	output_fputs(head_col, "NAME");
+	col_eoln(colp);
+	output_fprintf(body_col, "Project \"%s\"", project_name_get(pp)->str_text);
 	if (cstate_data->delta_number)
-		col_printf(body_col, ", Delta %ld", cstate_data->delta_number);
+		output_fprintf(body_col, ", Delta %ld", cstate_data->delta_number);
 	if (cstate_data->state < cstate_state_completed || option_verbose_get())
 	{
-		col_printf
+		output_fprintf
 		(
 			body_col,
 			", Change %ld",
 			magic_zero_decode(change_number)
 		);
 	}
-	col_puts(body_col, ".");
-	col_eoln();
+	output_fputc(body_col, '.');
+	col_eoln(colp);
 
 	/*
 	 * synopsis
 	 */
-	col_need(5);
-	col_puts(head_col, "SUMMARY");
-	col_eoln();
-	col_puts(body_col, cstate_data->brief_description->str_text);
-	col_eoln();
+	col_need(colp, 5);
+	output_fputs(head_col, "SUMMARY");
+	col_eoln(colp);
+	output_put_str(body_col, cstate_data->brief_description);
+	col_eoln(colp);
 
 	/*
 	 * description
 	 */
-	col_need(5);
-	col_puts(head_col, "DESCRIPTION");
-	col_eoln();
-	col_puts(body_col, cstate_data->description->str_text);
+	col_need(colp, 5);
+	output_fputs(head_col, "DESCRIPTION");
+	col_eoln(colp);
+	output_put_str(body_col, cstate_data->description);
 	if (cstate_data->test_exempt || cstate_data->test_baseline_exempt)
 	{
-		col_bol(body_col);
-		col_puts(body_col, "\n");
+		output_end_of_line(body_col);
+		output_fputc(body_col, '\n');
 		if (!cstate_data->regression_test_exempt)
 		{
-			col_puts
+			output_fputs
 			(
 				body_col,
 			       "This change must pass a full regression test.  "
@@ -170,7 +173,7 @@ list_change_details(project_name, change_number)
 		}
 		if (cstate_data->test_exempt)
 		{
-			col_puts
+			output_fputs
 			(
 				body_col,
        "This change is exempt from testing against the development directory.  "
@@ -178,14 +181,14 @@ list_change_details(project_name, change_number)
 		}
 		if (cstate_data->test_baseline_exempt)
 		{
-			col_puts
+			output_fputs
 			(
 				body_col,
 		      "This change is exempt from testing against the baseline."
 			);
 		}
 	}
-	col_eoln();
+	col_eoln(colp);
 
 	/*
 	 * show the sub-changes of a branch
@@ -193,28 +196,46 @@ list_change_details(project_name, change_number)
 	if (cstate_data->branch && option_verbose_get())
 	{
 		project_ty	*sub_pp;
-		int		number_col;
-		int		state_col;
-		int		description_col;
+		output_ty	*number_col;
+		output_ty	*state_col;
+		output_ty	*description_col;
 
-		col_need(5);
-		col_puts(head_col, "BRANCH CONTENTS");
-		col_eoln();
+		col_need(colp, 5);
+		output_fputs(head_col, "BRANCH CONTENTS");
+		col_eoln(colp);
 
 		/*
 		 * create the columns
 		 */
 		left = INDENT_WIDTH;
-		number_col = col_create(left, left + CHANGE_WIDTH);
+		number_col =
+			col_create
+			(
+				colp,
+				left,
+				left + CHANGE_WIDTH,
+				"Change\n-------"
+			);
 		left += CHANGE_WIDTH + 1;
-		col_heading(number_col, "Change\n-------");
 
-		state_col = col_create(left, left + STATE_WIDTH);
+		state_col =
+			col_create
+			(
+				colp,
+				left,
+				left + STATE_WIDTH,
+				"State\n-------"
+			);
 		left += STATE_WIDTH + 1;
-		col_heading(state_col, "State\n-------");
 
-		description_col = col_create(left, 0);
-		col_heading(description_col, "Description\n-------------");
+		description_col =
+			col_create
+			(
+				colp,
+				left,
+				0,
+				"Description\n-------------"
+			);
 
 		/*
 		 * list the sub changes
@@ -232,49 +253,49 @@ list_change_details(project_name, change_number)
 			change_bind_existing(sub_cp);
 
 			sub_cstate_data = change_cstate_get(sub_cp);
-			col_printf
+			output_fprintf
 			(
 				number_col,
 				"%4ld",
 				magic_zero_decode(sub_cn)
 			);
-			col_puts
+			output_fputs
 			(
 				state_col,
 				cstate_state_ename(sub_cstate_data->state)
 			);
 			if (cstate_data->brief_description)
 			{
-				col_puts
+				output_put_str
 				(
 					description_col,
-					sub_cstate_data->brief_description->str_text
+					sub_cstate_data->brief_description
 				);
 			}
-			col_eoln();
+			col_eoln(colp);
 			change_free(sub_cp);
 		}
-		col_heading(number_col, (char *)0);
-		col_heading(state_col, (char *)0);
-		col_heading(description_col, (char *)0);
+		output_delete(number_col);
+		output_delete(state_col);
+		output_delete(description_col);
 	}
-	col_eoln();
+	col_eoln(colp);
 
 	/*
 	 * architecture
 	 */
-	col_need(7);
-	col_printf
+	col_need(colp, 7);
+	output_fprintf
 	(
 		head_col,
 		"ARCHITECTURE%s",
 		(cstate_data->architecture->length == 1 ? "" : "S")
 	);
-	col_eoln();
-	col_puts(body_col, "This change must build and test in");
+	col_eoln(colp);
+	output_fputs(body_col, "This change must build and test in");
 	if (cstate_data->architecture->length > 1)
-		col_puts(body_col, " each of");
-	col_puts(body_col, " the");
+		output_fputs(body_col, " each of");
+	output_fputs(body_col, " the");
 	for (j = 0; j < cstate_data->architecture->length; ++j)
 	{
 		string_ty	*s;
@@ -283,19 +304,19 @@ list_change_details(project_name, change_number)
 		if (j)
 		{
 			if (j == cstate_data->architecture->length - 1)
-				col_puts(body_col, " and");
+				output_fputs(body_col, " and");
 			else
-				col_puts(body_col, ",");
+				output_fputs(body_col, ",");
 		}
-		col_printf(body_col, " \"%s\"",  s->str_text);
+		output_fprintf(body_col, " \"%s\"",  s->str_text);
 	}
-	col_printf
+	output_fprintf
 	(
 		body_col,
 		" architecture%s.",
 		(cstate_data->architecture->length == 1 ? "" : "s")
 	);
-	col_eoln();
+	col_eoln(colp);
 
 	if
 	(
@@ -304,38 +325,75 @@ list_change_details(project_name, change_number)
 		cstate_data->state == cstate_state_being_integrated
 	)
 	{
-		int	arch_col;
-		int	host_col;
-		int	build_col;
-		int	test_col;
-		int	test_bl_col;
-		int	test_reg_col;
+		output_ty	*arch_col;
+		output_ty	*host_col;
+		output_ty	*build_col;
+		output_ty	*test_col;
+		output_ty	*test_bl_col;
+		output_ty	*test_reg_col;
+		string_list_ty	done;
 
-		col_need(5);
+		string_list_constructor(&done);
+		col_need(colp, 5);
 		left = INDENT_WIDTH;
-		arch_col = col_create(left, left + ARCH_WIDTH);
+		arch_col =
+			col_create
+			(
+				colp,
+				left,
+				left + ARCH_WIDTH,
+				"arch.\n--------"
+			);
 		left += ARCH_WIDTH + 1;
-		col_heading(arch_col, "arch.\n--------");
 
-		host_col = col_create(left, left + HOST_WIDTH);
+		host_col =
+			col_create
+			(
+				colp,
+				left, left + HOST_WIDTH,
+				"host\n--------"
+			);
 		left += HOST_WIDTH + 1;
-		col_heading(host_col, "host\n--------");
 
-		build_col = col_create(left, left + TIME_WIDTH);
+		build_col =
+			col_create
+			(
+				colp,
+				left,
+				left + TIME_WIDTH,
+				"aeb\n---------"
+			);
 		left += TIME_WIDTH + 1;
-		col_heading(build_col, "aeb\n---------");
 
-		test_col = col_create(left, left + TIME_WIDTH);
+		test_col =
+			col_create
+			(
+				colp,
+				left,
+				left + TIME_WIDTH,
+				"aet\n---------"
+			);
 		left += TIME_WIDTH + 1;
-		col_heading(test_col, "aet\n---------");
 
-		test_bl_col = col_create(left, left + TIME_WIDTH);
+		test_bl_col =
+			col_create
+			(
+				colp,
+				left,
+				left + TIME_WIDTH,
+				"aet -bl\n---------"
+			);
 		left += TIME_WIDTH + 1;
-		col_heading(test_bl_col, "aet -bl\n---------");
 
-		test_reg_col = col_create(left, left + TIME_WIDTH);
+		test_reg_col =
+			col_create
+			(
+				colp,
+				left,
+				left + TIME_WIDTH,
+				"aet -reg\n---------"
+			);
 		left += TIME_WIDTH + 1;
-		col_heading(test_reg_col, "aet -reg\n---------");
 
 		for (j = 0; j < cstate_data->architecture->length; ++j)
 		{
@@ -345,9 +403,10 @@ list_change_details(project_name, change_number)
 			s = cstate_data->architecture->list[j];
 			tp = change_architecture_times_find(cp, s);
 			
-			col_puts(arch_col, tp->variant->str_text);
+			output_put_str(arch_col, tp->variant);
+			string_list_append(&done, tp->variant);
 			if (tp->node)
-				col_puts(host_col, tp->node->str_text);
+				output_put_str(host_col, tp->node);
 			showtime(build_col, tp->build_time, 0);
 			showtime
 			(
@@ -367,22 +426,34 @@ list_change_details(project_name, change_number)
 				tp->regression_test_time,
 				cstate_data->regression_test_exempt
 			);
-			col_eoln();
+			col_eoln(colp);
 		}
+		for (j = 0; j < cstate_data->architecture_times->length; ++j)
+		{
+			cstate_architecture_times tp;
 
-		col_heading(arch_col, (char *)0);
-		col_heading(host_col, (char *)0);
-		col_heading(build_col, (char *)0);
-		col_heading(test_col, (char *)0);
-		col_heading(test_bl_col, (char *)0);
-		col_heading(test_reg_col, (char *)0);
+			tp = cstate_data->architecture_times->list[j];
+			if (string_list_member(&done, tp->variant))
+				continue;
+			
+			output_put_str(arch_col, tp->variant);
+			string_list_append(&done, tp->variant);
+			if (tp->node)
+				output_put_str(host_col, tp->node);
+			showtime(build_col, tp->build_time, 1);
+			showtime(test_col, tp->test_time, 1);
+			showtime(test_bl_col, tp->test_baseline_time, 1);
+			showtime(test_reg_col, tp->regression_test_time, 1);
+			col_eoln(colp);
+		}
+		string_list_destructor(&done);
 
 		if (cstate_data->architecture->length > 1)
 		{
-			col_puts(build_col, "---------\n");
-			col_puts(test_col, "---------\n");
-			col_puts(test_bl_col, "---------\n");
-			col_puts(test_reg_col, "---------\n");
+			output_fputs(build_col, "---------\n");
+			output_fputs(test_col, "---------\n");
+			output_fputs(test_bl_col, "---------\n");
+			output_fputs(test_reg_col, "---------\n");
 
 			showtime(build_col, cstate_data->build_time, 0);
 			showtime
@@ -403,69 +474,101 @@ list_change_details(project_name, change_number)
 				cstate_data->regression_test_time,
 				cstate_data->regression_test_exempt
 			);
-			col_eoln();
+			col_eoln(colp);
 		}
+
+		output_delete(arch_col);
+		output_delete(host_col);
+		output_delete(build_col);
+		output_delete(test_col);
+		output_delete(test_bl_col);
+		output_delete(test_reg_col);
 	}
 
 	/*
 	 * cause
 	 */
-	col_need(5);
-	col_puts(head_col, "CAUSE");
-	col_eoln();
-	col_printf
+	col_need(colp, 5);
+	output_fputs(head_col, "CAUSE");
+	col_eoln(colp);
+	output_fprintf
 	(
 		body_col,
 		"This change was caused by %s.",
 		change_cause_ename(cstate_data->cause)
 	);
-	col_eoln();
+	col_eoln(colp);
 
 	/*
 	 * state
 	 */
 	if (cstate_data->state != cstate_state_completed)
 	{
-		col_need(5);
-		col_puts(head_col, "STATE");
-		col_eoln();
-		col_printf
+		col_need(colp, 5);
+		output_fputs(head_col, "STATE");
+		col_eoln(colp);
+		output_fprintf
 		(
 			body_col,
 			"This change is in the '%s' state.",
 			cstate_state_ename(cstate_data->state)
 		);
-		col_eoln();
+		col_eoln(colp);
 	}
 
 	/*
 	 * files
 	 */
-	col_need(5);
-	col_puts(head_col, "FILES");
-	col_eoln();
+	col_need(colp, 5);
+	output_fputs(head_col, "FILES");
+	col_eoln(colp);
 	if (change_file_nth(cp, (size_t)0))
 	{
-		int	usage_col;
-		int	action_col;
-		int	edit_col;
-		int	file_name_col;
+		output_ty	*usage_col;
+		output_ty	*action_col;
+		output_ty	*edit_col;
+		output_ty	*file_name_col;
 
 		left = INDENT_WIDTH;
-		usage_col = col_create(left, left + USAGE_WIDTH);
+		usage_col =
+			col_create
+			(
+				colp,
+				left,
+				left + USAGE_WIDTH,
+				"Type\n-------"
+			);
 		left += USAGE_WIDTH + 1;
-		col_heading(usage_col, "Type\n-------");
 
-		action_col = col_create(left, left + ACTION_WIDTH);
+		action_col =
+			col_create
+			(
+				colp,
+				left,
+				left + ACTION_WIDTH,
+				"Action\n--------"
+			);
 		left += ACTION_WIDTH + 1;
-		col_heading(action_col, "Action\n--------");
 
-		edit_col = col_create(left, left + EDIT_WIDTH);
+		edit_col =
+			col_create
+			(
+				colp,
+				left,
+				left + EDIT_WIDTH,
+				"Edit\n-------"
+			);
 		left += EDIT_WIDTH + 1;
-		col_heading(edit_col, "Edit\n-------");
 
-		file_name_col = col_create(left, 0);
-		col_heading(file_name_col, "File Name\n-----------");
+		file_name_col =
+			col_create
+			(
+				colp,
+				left,
+				0,
+				"File Name\n-----------"
+			);
+
 		for (j = 0; ; ++j)
 		{
 			fstate_src	src_data;
@@ -474,8 +577,8 @@ list_change_details(project_name, change_number)
 			if (!src_data)
 				break;
 			assert(src_data->file_name);
-			col_puts(usage_col, file_usage_ename(src_data->usage));
-			col_puts
+			output_fputs(usage_col, file_usage_ename(src_data->usage));
+			output_fputs
 			(
 				action_col,
 				file_action_ename(src_data->action)
@@ -501,101 +604,128 @@ list_change_details(project_name, change_number)
 						pp,
 						src_data->file_name
 					);
-				if (psrc_data && psrc_data->edit_number)
+				if (psrc_data && psrc_data->edit)
 				{
-					col_printf
+					assert(psrc_data->edit->revision);
+					output_fprintf
 					(
 						edit_col,
 						" (%s)",
-						psrc_data->edit_number->str_text
+					     psrc_data->edit->revision->str_text
 					);
 				}
 			}
-			if (src_data->edit_number_origin_new)
+			if (src_data->edit_origin_new)
 			{
 				/*
 				 * The ``cross branch merge'' version.
 				 */
-				col_bol(edit_col);
-				col_printf
+				assert(src_data->edit_origin_new->revision);
+				output_end_of_line(edit_col);
+				output_fprintf
 				(
 					edit_col,
 					"{cross %4s}",
-				      src_data->edit_number_origin_new->str_text
+				   src_data->edit_origin_new->revision->str_text
 				);
 			}
-			col_puts(file_name_col, src_data->file_name->str_text);
+			output_put_str(file_name_col, src_data->file_name);
 			if (src_data->move)
 			{
-				col_bol(file_name_col);
-				col_puts(file_name_col, "Moved ");
+				output_end_of_line(file_name_col);
+				output_fputs(file_name_col, "Moved ");
 				if (src_data->action == file_action_create)
-					col_puts(file_name_col, "from ");
+					output_fputs(file_name_col, "from ");
 				else
-					col_puts(file_name_col, "to ");
-				col_puts
+					output_fputs(file_name_col, "to ");
+				output_put_str
 				(
 					file_name_col,
-					src_data->move->str_text
+					src_data->move
 				);
 			}
-			col_eoln();
+			col_eoln(colp);
 		}
-		col_heading(usage_col, (char *)0);
-		col_heading(action_col, (char *)0);
-		col_heading(edit_col, (char *)0);
-		col_heading(file_name_col, (char *)0);
+		output_delete(usage_col);
+		output_delete(action_col);
+		output_delete(edit_col);
+		output_delete(file_name_col);
 	}
 	else
 	{
-		col_printf(body_col, "This change has no files.");
-		col_eoln();
+		output_fprintf(body_col, "This change has no files.");
+		col_eoln(colp);
 	}
 
 	/*
 	 * history
 	 */
-	col_need(5);
-	col_puts(head_col, "HISTORY");
-	col_eoln();
+	col_need(colp, 5);
+	output_fputs(head_col, "HISTORY");
+	col_eoln(colp);
 	if (option_verbose_get())
 	{
-		int	what_col;
-		int	when_col;
-		int	who_col;
-		int	why_col;
+		output_ty	*what_col;
+		output_ty	*when_col;
+		output_ty	*who_col;
+		output_ty	*why_col;
 
 		left = INDENT_WIDTH;
-		what_col = col_create(left, left + WHAT_WIDTH);
+		what_col =
+			col_create
+			(
+				colp,
+				left,
+				left + WHAT_WIDTH,
+				"What\n------"
+			);
 		left += WHAT_WIDTH + 1;
-		col_heading(what_col, "What\n------");
 
-		when_col = col_create(left, left + WHEN_WIDTH);
+		when_col =
+			col_create
+			(
+				colp,
+				left,
+				left + WHEN_WIDTH,
+				"When\n------"
+			);
 		left += WHEN_WIDTH + 1;
-		col_heading(when_col, "When\n------");
 
-		who_col = col_create(left, left + WHO_WIDTH);
+		who_col =
+			col_create
+			(
+				colp,
+				left,
+				left + WHO_WIDTH,
+				"Who\n-----"
+			);
 		left += WHO_WIDTH + 1;
-		col_heading(who_col, "Who\n-----");
 
-		why_col = col_create(left, 0);
-		col_heading(why_col, "Comment\n---------");
+		why_col =
+			col_create
+			(
+				colp,
+				left,
+				0,
+				"Comment\n---------"
+			);
+
 		for (j = 0; j < cstate_data->history->length; ++j)
 		{
 			cstate_history	history_data;
 			time_t		t;
 
 			history_data = cstate_data->history->list[j];
-			col_puts
+			output_fputs
 			(
 				what_col,
 				cstate_history_what_ename(history_data->what)
 			);
 			t = history_data->when;
-			col_puts(when_col, ctime(&t));
-			col_puts(who_col, history_data->who->str_text);
+			output_fputs(when_col, ctime(&t));
+			output_put_str(who_col, history_data->who);
 			if (history_data->why)
-				col_puts(why_col, history_data->why->str_text);
+				output_put_str(why_col, history_data->why);
 			if
 			(
 				history_data->what
@@ -614,8 +744,8 @@ list_change_details(project_name, change_number)
 					time(&finish);
 				if (finish - t >= ELAPSED_TIME_THRESHOLD)
 				{
-					col_bol(why_col);
-					col_printf
+					output_end_of_line(why_col);
+					output_fprintf
 					(
 						why_col,
 						"Elapsed time: %5.3f days.\n",
@@ -623,18 +753,18 @@ list_change_details(project_name, change_number)
 					);
 				}
 			}
-			col_eoln();
+			col_eoln(colp);
 		}
-		col_heading(what_col, (char *)0);
-		col_heading(when_col, (char *)0);
-		col_heading(who_col, (char *)0);
-		col_heading(why_col, (char *)0);
+		output_delete(what_col);
+		output_delete(when_col);
+		output_delete(who_col);
+		output_delete(why_col);
 	}
 	else
 	{
 		if (cstate_data->state >= cstate_state_being_developed)
 		{
-			col_printf
+			output_fprintf
 			(
 				body_col,
 				"Developed by %s.",
@@ -643,7 +773,7 @@ list_change_details(project_name, change_number)
 		}
 		if (cstate_data->state >= cstate_state_awaiting_integration)
 		{
-			col_printf
+			output_fprintf
 			(
 				body_col,
 				"  Reviewed by %s.",
@@ -652,20 +782,20 @@ list_change_details(project_name, change_number)
 		}
 		if (cstate_data->state >= cstate_state_being_integrated)
 		{
-			col_printf
+			output_fprintf
 			(
 				body_col,
 				"  Integrated by %s.",
 				change_integrator_name(cp)->str_text
 			);
 		}
-		col_eoln();
+		col_eoln(colp);
 	}
 
 	/*
 	 * clean up and go home
 	 */
-	col_close();
+	col_close(colp);
 	change_free(cp);
 	project_free(pp);
 	user_free(up);
