@@ -1,6 +1,6 @@
 /*
  *	aegis - project change supervisor
- *	Copyright (C) 1994, 1995, 1996 Peter Miller;
+ *	Copyright (C) 1994, 1995, 1996, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -118,8 +118,8 @@ lookup(vp, rhs, lvalue)
 		scp = sub_context_new();
 		rpt_value_free(rhs2);
 		trace((/*{*/"}\n"));
-		sub_var_set(scp, "Name1", "%s", vp->method->name);
-		sub_var_set(scp, "Name2", "%s", rhs->method->name);
+		sub_var_set_charstar(scp, "Name1", vp->method->name);
+		sub_var_set_charstar(scp, "Name2", rhs->method->name);
 		s = subst_intl(scp, i18n("illegal lookup ($name1[$name2])"));
 		sub_context_delete(scp);
 		result = rpt_value_error((void *)0, s);
