@@ -1,6 +1,6 @@
 /*
  *	aegis - project change supervisor
- *	Copyright (C) 1999, 2001 Peter Miller;
+ *	Copyright (C) 1999, 2001, 2002 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 #include <col/private.h>
 #include <error.h> /* for assert */
 #include <mem.h>
+#include <option.h>
 #include <output/to_wide.h>
 #include <trace.h>
 #include <wide_output/column.h>
@@ -623,6 +624,8 @@ eoln(fp)
 		this->need_to_emit_headers
 	&&
 		!wide_output_header_is_at_top_of_page(this->deeper)
+	&&
+		option_page_headers_get()
 	)
 		emit_header(this);
 
@@ -773,11 +776,9 @@ col_pretty_open(deeper, delete_on_close)
 	result = col_new(&vtbl);
 	this = (col_pretty_ty *)result;
 	this->deeper =
-		wide_output_header_open
-		(
-			wide_output_unexpand_open(deeper, delete_on_close, -1),
-			1
-		);
+		wide_output_unexpand_open(deeper, delete_on_close, -1);
+	if (option_page_headers_get())
+		this->deeper = wide_output_header_open(this->deeper, 1);
 	this->need_to_emit_headers = 0; /* not until one gets create()ed */
 	this->ncolumns = 0;
 	this->ncolumns_max = 0;
