@@ -1,6 +1,6 @@
 /*
  *	aegis - project change supervisor
- *	Copyright (C) 1999 Peter Miller;
+ *	Copyright (C) 1999, 2001 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -23,11 +23,20 @@
 #include <input/crlf.h>
 #include <input/file.h>
 #include <input/file_text.h>
+#include <str.h>
 
 
 input_ty *
 input_file_text_open(fn)
-	const char	*fn;
+	string_ty	*fn;
 {
 	return input_crlf(input_file_open(fn), 1);
 }
+
+
+void
+input_file_text_escaped_newline(ip)
+	input_ty	*ip;
+{
+	input_crlf_escaped_newline(ip);
+}
