@@ -1,7 +1,7 @@
 #!/bin/sh
 #
 #	aegis - project change supervisor
-#	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999 Peter Miller;
+#	Copyright (C) 1991-2001 Peter Miller;
 #	All rights reserved.
 #
 #	This program is free software; you can redistribute it and/or modify
@@ -84,7 +84,7 @@ check_it()
 {
 	sed	-e "s|$work|...|g" \
 		-e 's|= [0-9][0-9]*; /.*|= TIME;|' \
-		-e "s/$USER/USER/g" \
+		-e "s/\"$USER\"/\"USER\"/g" \
 		-e 's/19[0-9][0-9]/YYYY/' \
 		-e 's/20[0-9][0-9]/YYYY/' \
 		-e 's/crypto = ".*"/crypto = "GUNK"/' \
@@ -423,19 +423,31 @@ src =
 	{
 		file_name = "config";
 		action = create;
-		edit_number = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = source;
 	},
 	{
 		file_name = "main.c";
 		action = create;
-		edit_number = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = source;
 	},
 	{
 		file_name = "test/00/t0001a.sh";
 		action = create;
-		edit_number = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = test;
 	},
 ];
@@ -484,6 +496,7 @@ branch =
 	default_test_exemption = false;
 	skip_unlucky = false;
 	compress_database = false;
+	develop_end_action = goto_being_reviewed;
 	history =
 	[
 		{
@@ -526,8 +539,16 @@ src =
 	{
 		file_name = "config";
 		action = create;
-		edit_number = "1.1";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = source;
 		file_fp =
 		{
@@ -549,8 +570,16 @@ src =
 	{
 		file_name = "main.c";
 		action = create;
-		edit_number = "1.1";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = source;
 		file_fp =
 		{
@@ -572,8 +601,16 @@ src =
 	{
 		file_name = "test/00/t0001a.sh";
 		action = create;
-		edit_number = "1.1";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = test;
 		file_fp =
 		{
@@ -630,6 +667,7 @@ branch =
 	default_test_exemption = false;
 	skip_unlucky = false;
 	compress_database = false;
+	develop_end_action = goto_being_reviewed;
 	change =
 	[
 		1,
@@ -716,6 +754,7 @@ branch =
 	default_test_exemption = false;
 	skip_unlucky = false;
 	compress_database = false;
+	develop_end_action = goto_being_reviewed;
 	change =
 	[
 		1,
@@ -829,7 +868,11 @@ src =
 	{
 		file_name = "main.c";
 		action = modify;
-		edit_number_origin = "1.1";
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = source;
 	},
 ];
@@ -878,6 +921,7 @@ branch =
 	default_test_exemption = false;
 	skip_unlucky = false;
 	compress_database = false;
+	develop_end_action = goto_being_reviewed;
 	history =
 	[
 		{
