@@ -1,7 +1,7 @@
 #!/bin/sh
 #
 #	aegis - project change supervisor
-#	Copyright (C) 1995, 1996, 1997, 1998, 1999 Peter Miller;
+#	Copyright (C) 1995-2001 Peter Miller;
 #	All rights reserved.
 #
 #	This program is free software; you can redistribute it and/or modify
@@ -76,7 +76,7 @@ check_it()
 {
 	sed	-e "s|$work|...|g" \
 		-e 's|= [0-9][0-9]*; /.*|= TIME;|' \
-		-e "s/$USER/USER/g" \
+		-e "s/\"$USER\"/\"USER\"/g" \
 		-e 's/19[0-9][0-9]/YYYY/' \
 		-e 's/20[0-9][0-9]/YYYY/' \
 		-e 's/node = ".*";/node = "NODE";/' \
@@ -443,6 +443,7 @@ branch =
 	default_test_exemption = false;
 	skip_unlucky = false;
 	compress_database = false;
+	develop_end_action = goto_being_reviewed;
 	change =
 	[
 	];
@@ -526,6 +527,7 @@ branch =
 	default_test_exemption = false;
 	skip_unlucky = false;
 	compress_database = false;
+	develop_end_action = goto_being_reviewed;
 	change =
 	[
 	];
@@ -754,8 +756,16 @@ src =
 	{
 		file_name = "main.c";
 		action = create;
-		edit_number = "1.1";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = source;
 		locked_by = 1;
 		about_to_be_copied_by = 1;
@@ -843,6 +853,7 @@ branch =
 	default_test_exemption = false;
 	skip_unlucky = false;
 	compress_database = false;
+	develop_end_action = goto_being_reviewed;
 	history =
 	[
 		{
@@ -884,8 +895,16 @@ src =
 	{
 		file_name = "main.c";
 		action = modify;
-		edit_number = "1.2";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.2";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = source;
 		file_fp =
 		{
@@ -908,8 +927,16 @@ src =
 	{
 		file_name = "test/00/t0002a.sh";
 		action = create;
-		edit_number = "1.1";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = test;
 		file_fp =
 		{
@@ -953,8 +980,16 @@ src =
 	{
 		file_name = "config";
 		action = create;
-		edit_number = "1.1";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = source;
 		file_fp =
 		{
@@ -970,8 +1005,16 @@ src =
 	{
 		file_name = "main.c";
 		action = create;
-		edit_number = "1.1";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = source;
 		file_fp =
 		{
@@ -988,8 +1031,16 @@ src =
 	{
 		file_name = "test/00/t0001a.sh";
 		action = create;
-		edit_number = "1.1";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = test;
 		file_fp =
 		{
@@ -1046,8 +1097,16 @@ src =
 	{
 		file_name = "main.c";
 		action = modify;
-		edit_number = "1.2";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.2";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = source;
 		file_fp =
 		{
@@ -1070,8 +1129,16 @@ src =
 	{
 		file_name = "test/00/t0002a.sh";
 		action = create;
-		edit_number = "1.1";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = test;
 		file_fp =
 		{
@@ -1113,8 +1180,16 @@ src =
 	{
 		file_name = "main.c";
 		action = modify;
-		edit_number = "1.3";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.3";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = source;
 		test =
 		[
@@ -1125,8 +1200,16 @@ src =
 	{
 		file_name = "test/00/t0002a.sh";
 		action = create;
-		edit_number = "1.2";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.2";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = test;
 	},
 ];
@@ -1150,8 +1233,16 @@ src =
 	{
 		file_name = "config";
 		action = create;
-		edit_number = "1.1";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = source;
 		file_fp =
 		{
@@ -1167,8 +1258,16 @@ src =
 	{
 		file_name = "main.c";
 		action = create;
-		edit_number = "1.3";
-		edit_number_origin = "1.3";
+		edit =
+		{
+			revision = "1.3";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.3";
+			encoding = none;
+		};
 		usage = source;
 		file_fp =
 		{
@@ -1185,8 +1284,16 @@ src =
 	{
 		file_name = "test/00/t0001a.sh";
 		action = create;
-		edit_number = "1.1";
-		edit_number_origin = "1.1";
+		edit =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.1";
+			encoding = none;
+		};
 		usage = test;
 		file_fp =
 		{
@@ -1205,8 +1312,16 @@ src =
 	{
 		file_name = "test/00/t0002a.sh";
 		action = create;
-		edit_number = "1.2";
-		edit_number_origin = "1.2";
+		edit =
+		{
+			revision = "1.2";
+			encoding = none;
+		};
+		edit_origin =
+		{
+			revision = "1.2";
+			encoding = none;
+		};
 		usage = test;
 		file_fp =
 		{
